#include <Application.h>
#include <MediaRoster.h>
#include <MediaAddOn.h>
#include <MediaDefs.h>
#include <TimeSource.h>
#include <stdio.h>
#include <stdlib.h>

#define PRODUCER_NODE_NAME "SimpleSynth"

#define ERROR( err, msg ) \
	{ if ( err != B_OK ) {		\
		printf( "error: %s - %s\n", msg, strerror(err) );	\
		return -1; } }

int main(int argc, char** argv)
{
	status_t err;
	int32 count;
	BApplication app("application/x-vnd.media_node_test");
	
	BMediaRoster* roster = BMediaRoster::Roster(&err);	
	if ( roster == NULL || err != B_OK ) {
		printf("BMediaRoster::Roster() error. - %s\n", strerror(err) );
		return -1;
	}

//	Get producer node
	dormant_node_info info;
	count = 1;
	err = roster->GetDormantNodes( &info, &count, NULL, NULL, PRODUCER_NODE_NAME );
	ERROR( err, "BMediaRoster::GetDormantNode()" );

	printf( "%ld of %s node found\n", count, PRODUCER_NODE_NAME );

	media_node producer;
	err = roster->InstantiateDormantNode( info, &producer );
	ERROR( err, "BMediaRoster::InstantiateDormantNode()");

//	Get consumer node
	media_node consumer;
	err = roster->GetAudioMixer( &consumer );
	ERROR( err, "BMediaRoster::GetAudioMixer()");

//	Get end points
	media_input consumerInput;
	media_output producerOutput;
	count = 1;

	err = roster->GetFreeOutputsFor( producer, &producerOutput, 1, &count);
	ERROR( err, "BMediaRoster::GetFreeOutputsFor()");

	count = 1;
	err = roster->GetFreeInputsFor( consumer, &consumerInput, 1, &count);
	ERROR( err, "BMediaRoster::GetFreeInputsFor()");

//	Try connection
	media_format format;
	format.type = B_MEDIA_RAW_AUDIO;	
	format.u.raw_audio = media_raw_audio_format::wildcard;
	err = roster->Connect( 
			producerOutput.source, 
			consumerInput.destination, 
			&format, 
			&producerOutput, 
			&consumerInput);
	ERROR( err, "BMediaRoster::Connect()");

//	Set time source
	media_node time_source;
	err = roster->GetTimeSource( &time_source );
	ERROR( err, "BMediaRoster::GetTimeSource()");

	err = roster->SetTimeSourceFor( producer.node, time_source.node );
	ERROR( err, "BMediaRoster::SetTimeSourceFor()");

//	Now let's play it !
	bigtime_t startTime;
	err = roster->GetStartLatencyFor( time_source, &startTime );
	ERROR( err, "BMediaRoster::GetStartLatencyFor(timeSource)");	
	printf("start latency: %Ld\n", startTime ); 

	BTimeSource *bTimeSource;
	bTimeSource = roster->MakeTimeSourceFor( time_source );
	startTime += bTimeSource->PerformanceTimeFor( BTimeSource::RealTime() 
		+ 100 * 1000 );

	BMessenger messenger;
	err = roster->StartControlPanel( producer, &messenger );
	ERROR( err, "BMediaRoster::StartControlPanel(producer)");

	err = roster->StartNode( producer, startTime );
	ERROR( err, "BMediaRoster::StartNode(producer)");
	
	char s[10];
	fgets( s, 10, stdin );
	
	err = roster->StopNode( producer, 0, true );
	ERROR( err, "BMediaRoster::StopNode(producer)");

	messenger.SendMessage( B_QUIT_REQUESTED );

//	Clean-up
	err = roster->Disconnect(
			producer.node, producerOutput.source, 
			consumer.node, consumerInput.destination );
	ERROR( err, "BMediaRoster::Disconnect()");

	err = roster->ReleaseNode( producer );
	ERROR( err, "BMediaRoster::ReleaseNode(producer)");
	err = roster->ReleaseNode( consumer );
	ERROR( err, "BMediaRoster::ReleaseNode(consumer)");
	
	bTimeSource->Release();		
//	Never do this.
//	err = roster->ReleaseNode( timeSource );

	printf( "\n" );
	return 0;
}

