#include <MediaRoster.h>
#include <MediaTheme.h>
#include <stdio.h>

#include "NodeHarness.h"
#include "SimpleSynthAddOn.h"
#include "SimpleSynthNode.h"

#define ERROR( err, msg ) \
	{ if ( err != B_OK ) {		\
		fprintf( stderr, "error: %s - %s\n", msg, strerror(err) );	\
		return; } }

int main( int argc, char** argv )
{
	if ( argc != 2 ) {
		fprintf( stderr, "Was not launched by Media Server.\n" );
		return -1;
	}

	media_node_id id;
	sscanf( argv[1], "node=%ld", &id );

	NodeHarnessApp app( "application/x-vnd.simple_synth_addon", id );
	app.Run();
	
	return 0;
}

//	NodeHarnessApp

NodeHarnessApp::NodeHarnessApp(const char* signature, media_node_id id )
:	BApplication(signature),
	mNodeId( id )
{
}

void 
NodeHarnessApp::ReadyToRun()
{
	BWindow* win = new NodeHarnessWin( BRect(50,50,50,50), "SimpleSynth");

	status_t err;
	media_node clonedNode;

	BMediaRoster* roster = BMediaRoster::Roster();

	err = roster->GetNodeFor( mNodeId, &clonedNode );
	ERROR( err, "BMediaRoster::GetNodeFor()");

	BParameterWeb* web;
	err = roster->GetParameterWebFor( clonedNode, &web );
	ERROR( err, "BMediaRoster::GetParameterWebFor()");

	BView* view = BMediaTheme::ViewFor( web );
	win->AddChild(view);
	win->ResizeTo(view->Bounds().Width(), view->Bounds().Height() );

	win->Show();
}

//	NodeHarnessWin

NodeHarnessWin::
NodeHarnessWin( BRect frame, const char* title )
:	BWindow(frame, title, B_TITLED_WINDOW, B_ASYNCHRONOUS_CONTROLS)
{
}

NodeHarnessWin::
~NodeHarnessWin()
{
}

bool 
NodeHarnessWin::
QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

