#ifndef SIMPLE_SYNTH_NODE_H
#define SIMPLE_SYNTH_NODE_H

#include <BufferProducer.h>
#include <Controllable.h>
#include <MediaEventLooper.h>
#include <ParameterWeb.h>
#include <MediaRoster.h>
#include <BufferGroup.h>
#include <Buffer.h>
#include <TimeSource.h>

enum {
	GAIN_NULL_PARAM,
	GAIN_PARAM,
	PULSE_WIDTH_PARAM,
	MIDI_CHANNEL_PARAM
};

class SimpleSynthNode	
:	public BBufferProducer,
	public BControllable,
	public BMediaEventLooper
{
public:
	SimpleSynthNode(BMediaAddOn* addon);
	virtual ~SimpleSynthNode();

// BMediaNode methods
	BMediaAddOn* AddOn(	int32* internal_id) const;
	void NodeRegistered();
//	status_t StartControlPanel(BMessenger *outMessenger);

// BBufferProducer methods
	status_t FormatSuggestionRequested(
		media_type type,
		int32 quality,
		media_format* format);

	status_t FormatProposal(
		const media_source& output,
		media_format* format);

	status_t FormatChangeRequested(
		const media_source& source,
		const media_destination& destination,
		media_format* io_format,
		int32* _deprecated_);

	status_t GetNextOutput(
		int32* cookie,
		media_output* out_output);

	status_t DisposeOutputCookie(
		int32 cookie);

	status_t SetBufferGroup(
		const media_source& for_source,
		BBufferGroup* group);

	status_t PrepareToConnect(
		const media_source& what,
		const media_destination& where,
		media_format* format,
		media_source* out_source,
		char* out_name);

	void Connect(
		status_t error, 
		const media_source& source,
		const media_destination& destination,
		const media_format& format,
		char* io_name);

	void Disconnect(
		const media_source& what,
		const media_destination& where);

	void LateNoticeReceived(
		const media_source& what,
		bigtime_t how_much,
		bigtime_t performance_time);

	void EnableOutput(
		const media_source & what,
		bool enabled,
		int32* _deprecated_);

	status_t HandleMessage( 
		int32 message, 
		const void *data, 
		size_t size );

// BControllable methods
	virtual status_t GetParameterValue(
		int32 parameterID, 
		bigtime_t *lastChangeTime, 
		void *value, 
		size_t *ioSize);
	virtual void SetParameterValue(
		int32 parameterID, 
		bigtime_t changeTime,
		const void *value, 
		size_t size);

// BMediaEventLooper methods
	void HandleEvent(
		const media_timed_event* event,
		bigtime_t lateness,
		bool realTimeEvent = false);

// heart of the producer
protected:
	BBuffer* FillNextBuffer( bigtime_t eventTime );
	void FillBuffer( float* data );
	void AllocateBuffers();

// member variables
protected:

	media_format mPreferFormat;
	media_output mOutput;
	
	BMediaAddOn* mAddOn;
	BBufferGroup* mBufferGroup;

	bigtime_t mLatency, mInternalLatency;
	bool mOutputEnabled;
	bigtime_t mStartTime;
	size_t mFramesSent;

	float mGain;
	bigtime_t mGainLastChanged;
	
	int32 mMidiChannel;
	bigtime_t mMidiChannelChanged;

	float mPulseWidth;
	bigtime_t mPulseWidthChanged;

	double mLastPhase;
};

#endif
